==> nui_core-0.0.250.tar.gz: lib/ui/component/nui_screen_widget.dart <==
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:nui_core/nui_core.dart';

typedef ContentInflater = Widget Function(BuildContext context);
typedef ScreenStatusCallback = void Function();

void bindObserver({ScreenStatusCallback onResume, ScreenStatusCallback onPause, ScreenStatusCallback onInactive, ScreenStatusCallback onDetach, ScreenStatusCallback onSuspend}){
  WidgetsBinding.instance.addObserver(NUILifecycleEventHandler(
    resumeCallBack: onResume,
    pauseCallback: onPause,
    inactiveCallback: onInactive,
    detachCallback: onDetach,
    suspendCallback: onSuspend
  ));
}

void triggerNUIDialog(NUIDialogItem item){
  NUIBusEvent.get().send(NUIBusEventData<NUIDialogItem>(data: item, code: NUIScreen.EVENT_DIALOG), channel: NUIScreen.CHANNEL_SCREEN);
}

void triggerNUISnack(NUISnackItem item){
  NUIBusEvent.get().send(NUIBusEventData<NUISnackItem>(data: item, code: NUIScreen.EVENT_SNACK), channel: NUIScreen.CHANNEL_SCREEN);
}

void triggerNUIBottomSheet(NUIBottomSheetItem item){
  NUIBusEvent.get().send(NUIBusEventData<NUIBottomSheetItem>(data: item, code: NUIScreen.EVENT_BOTTOM_SHEET), channel: NUIScreen.CHANNEL_SCREEN);
}

class NUIScreen extends StatefulWidget {
  static const MODULE = "NUIScreen";
  static const EVENT_BOTTOM_SHEET = "NUIBusEventBottomSheet";
  static const EVENT_DIALOG = "NUIBusEventDialog";
  static const EVENT_SNACK = "NUIBusEventSnack";
  static const CHANNEL_SCREEN = "NUIChannelScreen";

  static List<String> _completedList = [];

  NUIScreen({@required this.content, @required this.screenCode, Key key, this.overlayStyle}): super(key : key);
  final ContentInflater content;
  final String screenCode;
  final NUIOverlayStyle overlayStyle;
  @override
  _NUIScreenState createState() => _NUIScreenState();
}

class _NUIScreenState extends State<NUIScreen> {

  String _getBusEventId(){
    return "NUIScreen-${widget.screenCode}";
  }

  @override
  void dispose(){
    super.dispose();
    NUINavigator.popContext();
    NUINavigator.popScreen(widget.screenCode);
    if(widget.overlayStyle != null) NUIDisplayUtil.reverseStatusBarStyleAttr();
    NUIBusEvent.get().disposeInvoke(_getBusEventId(), channel: NUIScreen.CHANNEL_SCREEN);
  }

  void _processBusEvent(BuildContext context, NUIBusEventData payload){
    logNUI(NUIScreen.MODULE, "Received bus event for code: ${payload.code} at screen ${widget.screenCode}");
    if(context == null) return;
    switch(payload.code){
      case NUIScreen.EVENT_BOTTOM_SHEET: {
        if(payload.data is NUIBottomSheetItem){
          if(NUIScreen._completedList.contains((payload.data as NUIBottomSheetItem).id)) return;
          NUIScreen._completedList.add((payload.data as NUIBottomSheetItem).id);
          final dialog = showNUIBottomSheetByItem(
              context: context,
              item: payload.data
          );
        }
        break;
      }
      case NUIScreen.EVENT_DIALOG: {
        if(payload.data is NUIDialogItem){
          if(NUIScreen._completedList.contains((payload.data as NUIDialogItem).id)) return;
          NUIScreen._completedList.add((payload.data as NUIDialogItem).id);
          final dialog = showNUIDialogByItem(
              context: context,
              item: payload.data
          );
        }
        break;
      }
      case NUIScreen.EVENT_SNACK: {
        if(payload.data is NUISnackItem){
          if(NUIScreen._completedList.contains((payload.data as NUISnackItem).id)) return;
          NUIScreen._completedList.add((payload.data as NUISnackItem).id);
          final dialog = snackByItem(
              context: context,
              item: payload.data
          );
        }
        break;
      }
    }
  }

  @override
  void initState(){
    super.initState();
    NUINavigator.addScreen(widget.screenCode);
    NUINavigator.addContext(() => context);
    bindObserver(onResume: (){
      logNUI(NUIScreen.MODULE, "onResume called on screen : ${widget.screenCode}");
    }, onSuspend: (){
      logNUI(NUIScreen.MODULE, "onSuspend called on screen : ${widget.screenCode}");
    });

    NUIBusEvent.get().listenInvoke(_getBusEventId(), channel: NUIScreen.CHANNEL_SCREEN, listener: (NUIBusEventData payload){
      _processBusEvent(context, payload);
    });
    WidgetsBinding.instance.addPostFrameCallback((_) {
      if(widget.overlayStyle != null) {
        NUIDisplayUtil.setStatusBarStyleAttr(widget.overlayStyle ?? getNUIApp().overlayStyle(context));
      }
      NUINavigator.setLatestScreenCallback((page, {duration, transition}) {
        NUINavigator.push(context, page, transition: transition);
      });
    });
  }

  @override
  Widget build(BuildContext context) {
    // return WillPopScope(
    //   child: widget.content(context),
    //   onWillPop: () async{
    //     logNUI(NUIScreen.MODULE, "Calling back on content ${widget.content.runtimeType}, code : ${widget.screenCode}");
    //     return true;
    //   });
    return widget.content(context);
  }
}

class NUILifecycleEventHandler extends WidgetsBindingObserver {
  final ScreenStatusCallback resumeCallBack;
  final ScreenStatusCallback suspendCallback;
  final ScreenStatusCallback inactiveCallback;
  final ScreenStatusCallback pauseCallback;
  final ScreenStatusCallback detachCallback;

  NUILifecycleEventHandler({
    this.resumeCallBack,
    this.suspendCallback,
    this.inactiveCallback,
    this.pauseCallback,
    this.detachCallback
  });

  void _triggerIfNotNull(ScreenStatusCallback callback){
    if(callback != null) callback();
  }

  @override
  Future<void> didChangeAppLifecycleState(AppLifecycleState state) async {
    logNUI(NUIScreen.MODULE, "State life cycle : $state");
    switch (state) {
      case AppLifecycleState.resumed: _triggerIfNotNull(resumeCallBack); break;
      case AppLifecycleState.inactive: _triggerIfNotNull(inactiveCallback); _triggerIfNotNull(suspendCallback); break;
      case AppLifecycleState.paused: _triggerIfNotNull(pauseCallback); _triggerIfNotNull(suspendCallback); break;
      case AppLifecycleState.detached: _triggerIfNotNull(detachCallback); _triggerIfNotNull(suspendCallback); break;
    }
  }
}

==> nui_core-0.1.1.tar.gz: lib/ui/component/nui_screen_widget.dart <==
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:nui_core/nui_core.dart';

typedef ContentInflater = Widget Function(BuildContext context);
typedef ScreenStatusCallback = void Function();

void bindObserver({ScreenStatusCallback onResume, ScreenStatusCallback onPause, ScreenStatusCallback onInactive, ScreenStatusCallback onDetach, ScreenStatusCallback onSuspend}){
  WidgetsBinding.instance.addObserver(NUILifecycleEventHandler(
    resumeCallBack: onResume,
    pauseCallback: onPause,
    inactiveCallback: onInactive,
    detachCallback: onDetach,
    suspendCallback: onSuspend
  ));
}

void triggerNUIDialog(NUIDialogItem item){
  NUIBusEvent.get().send(NUIBusEventData<NUIDialogItem>(data: item, code: NUIScreen.EVENT_DIALOG), channel: NUIScreen.CHANNEL_SCREEN);
}

void triggerNUISnack(NUISnackItem item){
  NUIBusEvent.get().send(NUIBusEventData<NUISnackItem>(data: item, code: NUIScreen.EVENT_SNACK), channel: NUIScreen.CHANNEL_SCREEN);
}

void triggerNUIBottomSheet(NUIBottomSheetItem item){
  NUIBusEvent.get().send(NUIBusEventData<NUIBottomSheetItem>(data: item, code: NUIScreen.EVENT_BOTTOM_SHEET), channel: NUIScreen.CHANNEL_SCREEN);
}

class NUIScreen extends StatefulWidget {
  static const MODULE = "NUIScreen";
  static const EVENT_BOTTOM_SHEET = "NUIBusEventBottomSheet";
  static const EVENT_DIALOG = "NUIBusEventDialog";
  static const EVENT_SNACK = "NUIBusEventSnack";
  static const CHANNEL_SCREEN = "NUIChannelScreen";

  static List<String> _completedList = List();

  NUIScreen({@required this.content, @required this.screenCode, Key key, this.overlayStyle}): super(key : key);
  final ContentInflater content;
  final String screenCode;
  final NUIOverlayStyle overlayStyle;
  @override
  _NUIScreenState createState() => _NUIScreenState();
}

class _NUIScreenState extends State<NUIScreen> {

  String _getBusEventId(){
    return "NUIScreen-${widget.screenCode}";
  }

  @override
  void dispose(){
    super.dispose();
    NUINavigator.popContext();
    NUINavigator.popScreen(widget.screenCode);
    if(widget.overlayStyle != null) NUIDisplayUtil.reverseStatusBarStyleAttr();
    NUIBusEvent.get().disposeInvoke(_getBusEventId(), channel: NUIScreen.CHANNEL_SCREEN);
  }

  void _processBusEvent(BuildContext context, NUIBusEventData payload){
    logNUI(NUIScreen.MODULE, "Received bus event for code: ${payload.code} at screen ${widget.screenCode}");
    if(context == null) return;
    switch(payload.code){
      case NUIScreen.EVENT_BOTTOM_SHEET: {
        if(payload.data is NUIBottomSheetItem){
          if(NUIScreen._completedList.contains((payload.data as NUIBottomSheetItem).id)) return;
          NUIScreen._completedList.add((payload.data as NUIBottomSheetItem).id);
          final dialog = showNUIBottomSheetByItem(
              context: context,
              item: payload.data
          );
        }
        break;
      }
      case NUIScreen.EVENT_DIALOG: {
        if(payload.data is NUIDialogItem){
          if(NUIScreen._completedList.contains((payload.data as NUIDialogItem).id)) return;
          NUIScreen._completedList.add((payload.data as NUIDialogItem).id);
          final dialog = showNUIDialogByItem(
              context: context,
              item: payload.data
          );
        }
        break;
      }
      case NUIScreen.EVENT_SNACK: {
        if(payload.data is NUISnackItem){
          if(NUIScreen._completedList.contains((payload.data as NUISnackItem).id)) return;
          NUIScreen._completedList.add((payload.data as NUISnackItem).id);
          final dialog = appNUI.snackByItem(
              context: context,
              item: payload.data
          );
        }
        break;
      }
    }
  }

  @override
  void initState(){
    super.initState();
    NUINavigator.addScreen(widget.screenCode);
    NUINavigator.addContext(() => context);
    bindObserver(onResume: (){
      logNUI(NUIScreen.MODULE, "onResume called on screen : ${widget.screenCode}");
    }, onSuspend: (){
      logNUI(NUIScreen.MODULE, "onSuspend called on screen : ${widget.screenCode}");
    });

    NUIBusEvent.get().listenInvoke(_getBusEventId(), channel: NUIScreen.CHANNEL_SCREEN, listener: (NUIBusEventData payload){
      _processBusEvent(context, payload);
    });
    if(widget.overlayStyle != null) {
      NUIDisplayUtil.setStatusBarStyleAttr(widget.overlayStyle ?? appNUI.overlayStyle(context));
    }
  }

  @override
  Widget build(BuildContext context) {
    NUINavigator.setLatestScreenCallback((page, {duration, transition}) {
      NUINavigator.push(context, page, transition: transition);
    });
    return WillPopScope(
      child: widget.content(context),
      onWillPop: () async{
        logNUI(NUIScreen.MODULE, "Calling back on content ${widget.content.runtimeType}, code : ${widget.screenCode}");
        return true;
      });
  }
}

class NUILifecycleEventHandler extends WidgetsBindingObserver {
  final ScreenStatusCallback resumeCallBack;
  final ScreenStatusCallback suspendCallback;
  final ScreenStatusCallback inactiveCallback;
  final ScreenStatusCallback pauseCallback;
  final ScreenStatusCallback detachCallback;

  NUILifecycleEventHandler({
    this.resumeCallBack,
    this.suspendCallback,
    this.inactiveCallback,
    this.pauseCallback,
    this.detachCallback
  });

  void _triggerIfNotNull(ScreenStatusCallback callback){
    if(callback != null) callback();
  }

  @override
  Future<void> didChangeAppLifecycleState(AppLifecycleState state) async {
    logNUI(NUIScreen.MODULE, "State life cycle : $state");
    switch (state) {
      case AppLifecycleState.resumed: _triggerIfNotNull(resumeCallBack); break;
      case AppLifecycleState.inactive: _triggerIfNotNull(inactiveCallback); _triggerIfNotNull(suspendCallback); break;
      case AppLifecycleState.paused: _triggerIfNotNull(pauseCallback); _triggerIfNotNull(suspendCallback); break;
      case AppLifecycleState.detached: _triggerIfNotNull(detachCallback); _triggerIfNotNull(suspendCallback); break;
    }
  }
}
